// Game Demo  - copyright Jim Roberts Pittsburgh, 15221  February 2014

/*
This program plays a simple game. 

The user tryies to place an aiming point over a moving square
using the mouse and click the botton to hit the target.

-The game offers three levels of difficulty.
-The aiming point is placed using easing so there is a lag between
    the acutal mouse position and the aiming point.
-The game is time to 30 seconds.
-The user has 10 tries to hit the target.


We will be using this code in class for several days.  
The code calls a number of "new" API functions -- LOOK THEM UP!!

Our first look at this code will be to examine different ways
of coding the if and if/else along with the logical AND and OR
operators.

We will also use the code as a pattern for planning and
coding larger and more complex programs.

The program provides ONE (and not the only one or necessairly
the preferred) way to:
- stage a program
- time a program
- score a user's play
- use keyPressed( ) and mousePressed( )
*/


// We use variables of the type float for locations and distances.
float x, y, dim, rad;
float deltaX, deltaY;
color hitFill, missFill, bgColor, figColor;

float easingCoefficient;

float aimX, aimY;

// We use variables of the type int to count things.
int startTime, currentTime, timeLimit, timeLapsed;
int tries, hits;



void setup( )
{
  size( 800, 600 );
  noCursor( );  // hide the cursor
  
  tries = 10;
  hits = 0;
  timeLimit = 30;
  
  textSize( 24 );
  textAlign( CENTER );
  
  rad = 25;  // corner radius of target square
  x = width/2;
  y = height/2;
   
  hitFill = color( 0, 255, 0 );
  missFill = color( 255, 0, 0 );
  bgColor = color( 0, 50 );
  figColor = color( 0, 0, 255 );
   
  showInstructions( );
}

void draw( )
{
  prepareWindow( );  
  moveFigure( );
  drawFigure( );
  setAimingPoint( );
  showScore( );
  checkTriesRemainingAndTime( );
}

// Called when the user tries to click on the moving target.
//   This needs some improvement -- the topic for Tuesday, February 11.
//   The control structure coded here works but it is difficult to modify
//   and we can do better.
void mousePressed ( )
{
   // A mousePressed event signifies a shot at the target
   tries--;
   if ( aimX >= x )  // Is mouse inside the left edge of the target?
   {
     if ( aimX <= x+dim )  // Is the mouse inside the right edge of the target?
     {
       if( aimY >= y )  // Is the mouse inside the top edge of the target?
       {
         if (aimY <= y+dim )  // Is the mouse inside the bottom edge of the target?
         {
           fill( hitFill );
           rect( 0, 0, width, height );
           dim--;  // Make the target smaller and harder to click.
           
           // Speed the target up to make it more challenging.
           // When the target is moving in a negative direction,
           // the ++ operator will actualliy slow it down so we
           // have to ask what direction it is moving
           
           if ( deltaX < 0 )  // Is the deltaX value negative?
           {
               deltaX--; 
           }
           else
          {
              deltaX++;
          }
           if ( deltaY < 0 )  // Is the deltaY value negative?
           {
               deltaY--; 
           }
           else
           {
               deltaX++;
           }

           hits++;
         }
         else
         {
           fill( missFill );
           rect( 0, 0, width, height );
         }
       }
       else
       {
         fill( missFill );
         rect( 0, 0, width, height );
       }
     }
     else
     {
       fill( missFill );
       rect( 0, 0, width, height );
     }
   } 
   else
   {
     fill( missFill );
     rect( 0, 0, width, height );
   }
}

void checkTriesRemainingAndTime( )
{
  if (tries <= 0 )  // Is the player out of tries?
  {
    background( 0 );
    showScore( );
    textSize( 60 );
    text( "GAME OVER", width/2, height/2 );
    noLoop( );  // stops iteration of draw( )
  } 
  else if ( timeLapsed >=timeLimit )  // Is the time up?
  {
    background( 0 );
    showScore( );
    textSize( 60 );
    text( "TIME'S UP", width/2, height/2 );
    noLoop( ); // stops iteration of draw( ) 
  } 
}

void showScore( )
{
  fill( 0, 255, 0 );
  text( "Hits: " + hits, width/2, height-20 );
  text( "Tries remaining: " + tries, width/2, height-50 ); 
  
  currentTime = millis( );
  timeLapsed = (currentTime - startTime)/1000;  // covert to seconds
  text( "Time: " + (timeLimit-timeLapsed), width/2, height - 80 );
 
}

// Uses easing to determine the aiming point so aiming point lags 
// behind the mouse position.
void setAimingPoint( )
{
   float xDist = mouseX - aimX;
   float yDist = mouseY - aimY;
   aimX += xDist * easingCoefficient;
   aimY += yDist * easingCoefficient;
   
   stroke( 100, 100, 0 );
   strokeWeight( 1 );
   
   line( 0, 0, aimX-15, aimY-15 );
   line( 0, height, aimX-15, aimY+15 );
   line( width, 0, aimX+15, aimY-15 );
   line( width, height, aimX+15, aimY+15 );
   
      // Is the aiming point is over the target??
   if ( aimX >= x && aimX <= x+dim &&
        aimY >= y && aimY <= y+dim )
   {
     fill( 0, 255, 0 );
   }
   else  // The aiming point is not over the target
   {
     fill( 255, 0, 0 );
   }
   rect( aimX-15, aimY-15, 30, 30 );
}

void drawFigure( )
{
   fill( figColor );
   rect( x, y, dim, dim, rad );
}

//  There is redundancy here -- we can do better...
void moveFigure( )
{
  x += deltaX;
  if (x > width-dim )  // Is the target too far right?
  {
     deltaX = -deltaX;  
  }
  if (x < 0 )          // Is the target too far left?
  {
    deltaX = -deltaX;  
  }
  
  y += deltaY; 
  if (y > height-dim ) // Is the target too high?
  { 
    deltaY = -deltaY; 
  } 
  if (y < 0 )         // Is the target too low?
  {
    deltaY = -deltaY;
  }
}

void prepareWindow( )
{
   noStroke( );
   fill( bgColor );
   rect( 0, 0, width, height ); 
}

void showInstructions( )
{
   background( 0 );
   fill( 255 );
   text("Hit the Square", width/2, 50 );
   text("You have "+ tries + " tries and " + timeLimit + " seconds to hit the square.", width/2, 80 );
   text("How good are you??? -- Give it a try!!!", width/2, 110 );
   text("Press [D] for a difficult game.", width/2, 180);
   text("Press [M] for a less difficult game.", width/2, 210 );
   text("Press [W] for a wimpy game.", width/2, 240 );
   noLoop (); //stops iteration of draw
}


// Main taks for keyPressed( ) is to initilize variables based on user input
// and to start the game after the user has read the instructions.
void keyPressed( )
{
    // Was the d or the D key pressed? ==> make it a difficult game
   if ( key == 'D' || key == 'd' )
   {
      dim = random( 20,  50 );  
      
      deltaX = random( 5, 10 );
      deltaY = random( 5, 10 );
      
      easingCoefficient = .03;    
   }
     // Was the m or the M key pressed?  ==>> make it a moderate game
   else if ( key == 'M' || key == 'm' )
   {
      dim = random( 50,  70 );  
      
      deltaX = random( 3, 8 );
      deltaY = random( 3, 8 );
      
      easingCoefficient = .03;        
   }
     //  The d/D/m/M keys were no pressed... ==> make it an easy game
   else
   {
      dim = random( 70,  130 );  

      deltaX = random( 1, 5 );
      deltaY = random( 1, 5 );
      
      easingCoefficient = .05;     
   }
   
   loop( );  // re-stars iteration of draw( )
   startTime = millis( );  // start the clock
}

